/**
 @file
 sftp - upload of files with sftp from chilkat (http://www.chilkatsoft.com/)
 johannes schmidt - johannes.schmidt@fosterkent.com	
 
 @ingroup	motionmixes
 */

#include "ext.h"
#include "ext_obex.h"
#include "ext_strings.h"
#include "ext_common.h"
#include "ext_systhread.h"

#include "CkSFtp.h"
using namespace std;

// a wrapper for cpost() only called for debug builds on Windows
// to see these console posts, run the DbgView program (part of the SysInternals package distributed by Microsoft)
#if defined( NDEBUG ) || defined( MAC_VERSION )
#define DPOST
#else
#define DPOST cpost
#endif

// a macro to mark exported symbols in the code without requiring an external file to define them
#ifdef WIN_VERSION
	// note that this is the required syntax on windows regardless of whether the compiler is msvc or gcc
	#define T_EXPORT __declspec(dllexport)
#else // MAC_VERSION
	// the mac uses the standard gcc syntax, you should also set the -fvisibility=hidden flag to hide the non-marked symbols
	#define T_EXPORT __attribute__((visibility("default")))
#endif


// max object instance data
typedef struct _sftp {
	t_object			c_box;
	CkSFtp				*sftp;
	void				*c_outlet;
	t_systhread_mutex	c_mutex;
	t_atom				resultList[2];
} t_sftp;


// prototypes
void*	sftp_new(t_symbol *s, long argc, t_atom *argv);
void	sftp_free(t_sftp* x);
// sftp
void	sftp_connect(t_sftp *x, t_symbol *msg, long argc, t_atom *argv);
void	sftp_disconnect(t_sftp *x);
void	sftp_upload(t_sftp *x, t_symbol *msg, long argc, t_atom *argv);

// globals
static t_class	*s_sftp_class = NULL;

/************************************************************************************/

int T_EXPORT main(void)
{
	t_class	*c = class_new("sftp",
							(method)sftp_new,
							(method)sftp_free,
							sizeof(t_sftp),
							(method)NULL,
							A_GIMME,
							0);

	common_symbols_init();
	class_addmethod(c, (method)sftp_connect,	"connect",		A_GIMME,0);
	class_addmethod(c, (method)sftp_disconnect,	"disconnect"	,0);
	class_addmethod(c, (method)sftp_upload,		"upload",		A_GIMME,0);
	class_register(_sym_box, c);
	s_sftp_class = c;

	return 0;
}


/************************************************************************************/
// Object Creation Method

void *sftp_new(t_symbol *s, long argc, t_atom *argv)
{
	t_sftp	*x;

	x = (t_sftp*)object_alloc(s_sftp_class);
	if (x) {
		systhread_mutex_new(&x->c_mutex, 0);
		x->c_outlet = outlet_new(x, NULL);
	}
	return(x);
}


void sftp_free(t_sftp *x)
{
	systhread_mutex_free(x->c_mutex);
	delete x->sftp;
}


/************************************************************************************/
// Methods bound to input/inlets

void sftp_do_upload(t_sftp *x, t_symbol *msg, long argc, t_atom *argv)
{
	bool success;
	
	
	if (argc < 2){
		post("sftp_do_upload: too less arguments to upload file");
		atom_setsym(x->resultList+0,gensym("status"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);		
        return;		
	}
		
	if(x->sftp == NULL){
		post("sftp_do_upload : pointer = NULL");
		atom_setsym(x->resultList+0,gensym("status"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);
		return;
	}

	t_symbol *localfile = atom_getsym(argv);
	t_symbol *remotefile = atom_getsym(argv+1);
	
    //  Upload from the local file to the SSH server.
    //  Important -- the remote filepath is the 1st argument,
    //  the local filepath is the 2nd argument;
	
	char localConformFilePath[MAX_PATH_CHARS];
	//PATH_TYPE_ABSOLUTE
	//PATH_TYPE_BOOT
	
	path_nameconform(localfile->s_name, localConformFilePath, PATH_STYLE_SLASH, PATH_TYPE_BOOT);

	//start cpp thread safety
	systhread_mutex_lock(x->c_mutex);	
	
    success = x->sftp->UploadFileByName(remotefile->s_name,localConformFilePath);
    if (success != true) {
		post("FEHLER BEI UPLOAD");
        post("%s\n",x->sftp->lastErrorText());
		systhread_mutex_unlock(x->c_mutex);		
		atom_setsym(x->resultList+0,gensym("status"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);
        return;
    }	
	systhread_mutex_unlock(x->c_mutex);
	atom_setsym(x->resultList+0,gensym("status"));	//result-type	
	atom_setlong(x->resultList+1,1);				//status 0 = nok
	outlet_list(x->c_outlet,0L,2,x->resultList);	
}

void sftp_upload(t_sftp *x, t_symbol *msg, long argc, t_atom *argv)
{
	defer(x, (method)sftp_do_upload, msg, argc, argv);
}




/************************************************************************************/
void sftp_do_connect(t_sftp *x, t_symbol *msg, long argc, t_atom *argv)
{	
    bool success;
	post("sftp::ya estamos en CONNECT");
	if (argc < 3){
		post("sftp_do_connect: too less arguments to establish connection");
		atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);		
        return;		
	}
	
	t_symbol *url = atom_getsym(argv);
	t_symbol *usr = atom_getsym(argv+1);
	t_symbol *pwd = atom_getsym(argv+2);	
	
	systhread_mutex_lock(x->c_mutex);
	if(x->sftp == NULL){
		post("sftp::create new pointer");
		x->sftp = new CkSFtp();
	}

	//  unlock the component with the purchased code
	
    //success = x->sftp->UnlockComponent("FOSTERSSH_zFbw8mmQ1QnX");
	success = x->sftp->UnlockComponent("Anything for 30-day trial");
    if (success != true) {
        post("%s\n",x->sftp->lastErrorText());
		systhread_mutex_unlock(x->c_mutex);
		atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);		
        return;
    }
	post("sftp::unlock successfull");
	
    //  Set some timeouts, in milliseconds:
    x->sftp->put_ConnectTimeoutMs(15000);
    x->sftp->put_IdleTimeoutMs(15000);
	
    long port;
    port = 22;
	
    success = x->sftp->Connect(url->s_name,port);
    if (success != true) {
        post("%s\n",x->sftp->lastErrorText());
		systhread_mutex_unlock(x->c_mutex);
		atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 0 = nok
		outlet_list(x->c_outlet,0L,2,x->resultList);		
        return;
    }
	post("sftp::Connect successfull");
	
    //  Authenticate with the SSH server.  Chilkat SFTP supports
    //  both password-based authenication as well as public-key
    //  authentication.  This example uses password authenication.
	success = x->sftp->AuthenticatePw(usr->s_name,pwd->s_name);

    if (success != true) {
        post("%s\n",x->sftp->lastErrorText());
		systhread_mutex_unlock(x->c_mutex);
		atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 1 = ok
		outlet_list(x->c_outlet,0L,2,x->resultList);		
        return;
    }
	post("sftp::Authenticate successfull");
	
    //  After authenticating, the SFTP subsystem must be initialized:
    success = x->sftp->InitializeSftp();
    if (success != true) {
        post("%s\n",x->sftp->lastErrorText());
		systhread_mutex_unlock(x->c_mutex);
		atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
		atom_setlong(x->resultList+1,0);				//status 1 = ok
		outlet_list(x->c_outlet,0L,2,x->resultList);
        return;
    }
	post("sftp::Init successfull");
	
	systhread_mutex_unlock(x->c_mutex);
	

	atom_setsym(x->resultList+0,gensym("connect"));	//result-type	
	atom_setlong(x->resultList+1,1);				//status 1 = ok
	outlet_list(x->c_outlet,0L,2,x->resultList);
}

void sftp_connect(t_sftp *x, t_symbol *msg, long argc, t_atom *argv)
{
	defer(x, (method)sftp_do_connect, msg, argc, argv);
}




/************************************************************************************/
void sftp_do_disconnect(t_sftp *x)
{
	systhread_mutex_lock(x->c_mutex);
	x->sftp->Disconnect();
	//post("sftp::sftp_do_disconnect successfull");
	atom_setsym(x->resultList+0,gensym("disconnect"));	//result-type	
	atom_setlong(x->resultList+1,1);					//status 1 = ok
	outlet_list(x->c_outlet,0L,2,x->resultList);	
	systhread_mutex_unlock(x->c_mutex);
}

void sftp_disconnect(t_sftp *x)
{
	defer(x, (method)sftp_do_disconnect, 0, 0, 0);
}

